@@ -8,7 +8,10 @@ namespace ast {
 
 class Printer : public Visitor {
 public:
-  explicit Printer(std::ostream &out) : out_(out) { }
+  explicit Printer(std::ostream &out, bool print_types = false)
+      : out_(out), print_types(print_types)
+  {
+  }
 
   void visit(Integer &integer) override;
   void visit(PositionalParameter &param) override;
@@ -25,11 +28,14 @@ public:
   void visit(FieldAccess &acc) override;
   void visit(ArrayAccess &arr) override;
   void visit(Cast &cast) override;
+  void visit(Tuple &tuple) override;
   void visit(ExprStatement &expr) override;
   void visit(AssignMapStatement &assignment) override;
   void visit(AssignVarStatement &assignment) override;
   void visit(If &if_block) override;
   void visit(Unroll &unroll) override;
+  void visit(While &while_block) override;
+  void visit(Jump &jump) override;
   void visit(Predicate &pred) override;
   void visit(AttachPoint &ap) override;
   void visit(Probe &probe) override;
@@ -39,6 +45,8 @@ public:
 
 private:
   std::ostream &out_;
+  bool print_types = false;
+  std::string type(const SizedType &ty);
 };
 
 } // namespace ast
