@@ -31,24 +31,31 @@ void check_kprobe(Probe &p,
   EXPECT_EQ(func_offset, p.func_offset);
 }
 
-static const std::string uprobe_name(const std::string &path, const std::string &attach_point,
-                                     uint64_t address, uint64_t func_offset)
+static const std::string uprobe_name(const std::string &path,
+                                     const std::string &attach_point,
+                                     uint64_t address,
+                                     uint64_t func_offset,
+                                     bool retprobe = false)
 {
+  auto provider = retprobe ? "uretprobe:" : "uprobe:";
   if (attach_point.empty()) {
-    return "uprobe:" + path + ":" + std::to_string(address);
+    return provider + path + ":" + std::to_string(address);
   } else {
     auto str = func_offset ? "+" + std::to_string(func_offset) : "";
-    return "uprobe:" + path + ":" + attach_point + str;
+    return provider + path + ":" + attach_point + str;
   }
 }
 
 void check_uprobe(Probe &p, const std::string &path, const std::string &attach_point, const std::string &orig_name,
                   uint64_t address = 0, uint64_t func_offset = 0)
 {
-  EXPECT_EQ(ProbeType::uprobe, p.type);
+  bool retprobe = orig_name.find("uretprobe:") == 0 ||
+                  orig_name.find("ur:") == 0;
+  EXPECT_EQ(retprobe ? ProbeType::uretprobe : ProbeType::uprobe, p.type);
   EXPECT_EQ(attach_point, p.attach_point);
   EXPECT_EQ(orig_name, p.orig_name);
-  EXPECT_EQ(uprobe_name(path, attach_point, address, func_offset), p.name);
+  EXPECT_EQ(uprobe_name(path, attach_point, address, func_offset, retprobe),
+            p.name);
   EXPECT_EQ(address, p.address);
   EXPECT_EQ(func_offset, p.func_offset);
 }
@@ -111,7 +118,8 @@ void check_special_probe(Probe &p, const std::string &attach_point, const std::s
 
 TEST(bpftrace, add_begin_probe)
 {
-  ast::AttachPoint a("BEGIN");
+  ast::AttachPoint a("");
+  a.provider = "BEGIN";
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -125,7 +133,8 @@ TEST(bpftrace, add_begin_probe)
 
 TEST(bpftrace, add_end_probe)
 {
-  ast::AttachPoint a("END");
+  ast::AttachPoint a("");
+  a.provider = "END";
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -139,7 +148,9 @@ TEST(bpftrace, add_end_probe)
 
 TEST(bpftrace, add_probes_single)
 {
-  ast::AttachPoint a("kprobe", "sys_read");
+  ast::AttachPoint a("");
+  a.provider = "kprobe";
+  a.func = "sys_read";
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -153,8 +164,12 @@ TEST(bpftrace, add_probes_single)
 
 TEST(bpftrace, add_probes_multiple)
 {
-  ast::AttachPoint a1("kprobe", "sys_read");
-  ast::AttachPoint a2("kprobe", "sys_write");
+  ast::AttachPoint a1("");
+  a1.provider = "kprobe";
+  a1.func = "sys_read";
+  ast::AttachPoint a2("");
+  a2.provider = "kprobe";
+  a2.func = "sys_write";
   ast::AttachPointList attach_points = { &a1, &a2 };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -170,9 +185,16 @@ TEST(bpftrace, add_probes_multiple)
 
 TEST(bpftrace, add_probes_wildcard)
 {
-  ast::AttachPoint a1("kprobe", "sys_read");
-  ast::AttachPoint a2("kprobe", "my_*");
-  ast::AttachPoint a3("kprobe", "sys_write");
+  ast::AttachPoint a1("");
+  a1.provider = "kprobe";
+  a1.func = "sys_read";
+  ast::AttachPoint a2("");
+  a2.provider = "kprobe";
+  a2.func = "my_*";
+  a2.need_expansion = true;
+  ast::AttachPoint a3("");
+  a3.provider = "kprobe";
+  a3.func = "sys_write";
   ast::AttachPointList attach_points = { &a1, &a2, &a3 };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -195,9 +217,16 @@ TEST(bpftrace, add_probes_wildcard)
 
 TEST(bpftrace, add_probes_wildcard_no_matches)
 {
-  ast::AttachPoint a1("kprobe", "sys_read");
-  ast::AttachPoint a2("kprobe", "not_here_*");
-  ast::AttachPoint a3("kprobe", "sys_write");
+  ast::AttachPoint a1("");
+  a1.provider = "kprobe";
+  a1.func = "sys_read";
+  ast::AttachPoint a2("");
+  a2.provider = "kprobe";
+  a2.func = "not_here_*";
+  a2.need_expansion = true;
+  ast::AttachPoint a3("");
+  a3.provider = "kprobe";
+  a3.func = "sys_write";
   ast::AttachPointList attach_points = { &a1, &a2, &a3 };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -216,10 +245,53 @@ TEST(bpftrace, add_probes_wildcard_no_matches)
   check_kprobe(bpftrace->get_probes().at(1), "sys_write", probe_orig_name);
 }
 
+TEST(bpftrace, add_probes_kernel_module)
+{
+  ast::AttachPoint a1("");
+  a1.provider = "kprobe";
+  a1.func = "func_in_mod";
+  ast::AttachPointList attach_points = { &a1 };
+  ast::Probe probe(&attach_points, nullptr, nullptr);
+
+  auto bpftrace = get_strict_mock_bpftrace();
+  ASSERT_EQ(0, bpftrace->add_probe(probe));
+  ASSERT_EQ(1U, bpftrace->get_probes().size());
+  ASSERT_EQ(0U, bpftrace->get_special_probes().size());
+
+  std::string probe_orig_name = "kprobe:func_in_mod";
+  check_kprobe(bpftrace->get_probes().at(0), "func_in_mod", probe_orig_name);
+}
+
+TEST(bpftrace, add_probes_kernel_module_wildcard)
+{
+  ast::AttachPoint a1("");
+  a1.provider = "kprobe";
+  a1.func = "func_in_mo*";
+  a1.need_expansion = true;
+  ast::AttachPointList attach_points = { &a1 };
+  ast::Probe probe(&attach_points, nullptr, nullptr);
+
+  auto bpftrace = get_strict_mock_bpftrace();
+  EXPECT_CALL(*bpftrace,
+              get_symbols_from_file(
+                  "/sys/kernel/debug/tracing/available_filter_functions"))
+      .Times(1);
+
+  ASSERT_EQ(0, bpftrace->add_probe(probe));
+  ASSERT_EQ(1U, bpftrace->get_probes().size());
+  ASSERT_EQ(0U, bpftrace->get_special_probes().size());
+
+  std::string probe_orig_name = "kprobe:func_in_mo*";
+  check_kprobe(bpftrace->get_probes().at(0), "func_in_mod", probe_orig_name);
+}
+
 TEST(bpftrace, add_probes_offset)
 {
   uint64_t offset = 10;
-  ast::AttachPoint a("kprobe", "sys_read", offset);
+  ast::AttachPoint a("");
+  a.provider = "kprobe";
+  a.func = "sys_read";
+  a.func_offset = offset;
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -235,7 +307,10 @@ TEST(bpftrace, add_probes_offset)
 
 TEST(bpftrace, add_probes_uprobe)
 {
-  ast::AttachPoint a("uprobe", "/bin/sh", "foo", true);
+  ast::AttachPoint a("");
+  a.provider = "uprobe";
+  a.target = "/bin/sh";
+  a.func = "foo";
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -249,7 +324,11 @@ TEST(bpftrace, add_probes_uprobe)
 
 TEST(bpftrace, add_probes_uprobe_wildcard)
 {
-  ast::AttachPoint a("uprobe", "/bin/sh", "*open", true);
+  ast::AttachPoint a("");
+  a.provider = "uprobe";
+  a.target = "/bin/sh";
+  a.func = "*open";
+  a.need_expansion = true;
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -267,7 +346,11 @@ TEST(bpftrace, add_probes_uprobe_wildcard)
 
 TEST(bpftrace, add_probes_uprobe_wildcard_no_matches)
 {
-  ast::AttachPoint a("uprobe", "/bin/sh", "foo*", true);
+  ast::AttachPoint a("");
+  a.provider = "uprobe";
+  a.target = "/bin/sh";
+  a.func = "foo*";
+  a.need_expansion = true;
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -281,7 +364,10 @@ TEST(bpftrace, add_probes_uprobe_wildcard_no_matches)
 
 TEST(bpftrace, add_probes_uprobe_string_literal)
 {
-  ast::AttachPoint a("uprobe", "/bin/sh", "foo*", false);
+  ast::AttachPoint a("");
+  a.provider = "uprobe";
+  a.target = "/bin/sh";
+  a.func = "foo*";
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -295,7 +381,10 @@ TEST(bpftrace, add_probes_uprobe_string_literal)
 
 TEST(bpftrace, add_probes_uprobe_address)
 {
-  ast::AttachPoint a("uprobe", "/bin/sh", 1024);
+  ast::AttachPoint a("");
+  a.provider = "uprobe";
+  a.target = "/bin/sh";
+  a.address = 1024;
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -309,7 +398,11 @@ TEST(bpftrace, add_probes_uprobe_address)
 
 TEST(bpftrace, add_probes_uprobe_string_offset)
 {
-  ast::AttachPoint a("uprobe", "/bin/sh", "foo", (uint64_t) 10);
+  ast::AttachPoint a("");
+  a.provider = "uprobe";
+  a.target = "/bin/sh";
+  a.func = "foo";
+  a.func_offset = 10;
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -321,9 +414,88 @@ TEST(bpftrace, add_probes_uprobe_string_offset)
   check_uprobe(bpftrace.get_probes().at(0), "/bin/sh", "foo", "uprobe:/bin/sh:foo+10", 0, 10);
 }
 
+TEST(bpftrace, add_probes_uprobe_cpp_symbol)
+{
+  for (auto &provider : { "uprobe", "uretprobe" })
+  {
+    ast::AttachPoint a("");
+    a.provider = provider;
+    a.target = "/bin/sh";
+    a.func = "cpp_mangled";
+    a.need_expansion = true;
+    ast::AttachPointList attach_points = { &a };
+    ast::Probe probe(&attach_points, nullptr, nullptr);
+
+    auto bpftrace = get_strict_mock_bpftrace();
+    EXPECT_CALL(*bpftrace, extract_func_symbols_from_path("/bin/sh")).Times(1);
+
+    ASSERT_EQ(0, bpftrace->add_probe(probe));
+    ASSERT_EQ(2U, bpftrace->get_probes().size());
+    ASSERT_EQ(0U, bpftrace->get_special_probes().size());
+    auto orig_name = std::string(provider) + ":/bin/sh:cpp_mangled";
+    check_uprobe(
+        bpftrace->get_probes().at(0), "/bin/sh", "_Z11cpp_mangledi", orig_name);
+    check_uprobe(
+        bpftrace->get_probes().at(1), "/bin/sh", "_Z11cpp_mangledv", orig_name);
+  }
+}
+
+TEST(bpftrace, add_probes_uprobe_cpp_symbol_full)
+{
+  ast::AttachPoint a("");
+  a.provider = "uprobe";
+  a.target = "/bin/sh";
+  a.func = "cpp_mangled(int)";
+  a.need_expansion = true;
+  ast::AttachPointList attach_points = { &a };
+  ast::Probe probe(&attach_points, nullptr, nullptr);
+
+  auto bpftrace = get_strict_mock_bpftrace();
+  EXPECT_CALL(*bpftrace, extract_func_symbols_from_path("/bin/sh")).Times(1);
+
+  ASSERT_EQ(0, bpftrace->add_probe(probe));
+  ASSERT_EQ(1U, bpftrace->get_probes().size());
+  ASSERT_EQ(0U, bpftrace->get_special_probes().size());
+  check_uprobe(bpftrace->get_probes().at(0),
+               "/bin/sh",
+               "_Z11cpp_mangledi",
+               "uprobe:/bin/sh:cpp_mangled(int)");
+}
+
+TEST(bpftrace, add_probes_uprobe_cpp_symbol_wildcard)
+{
+  ast::AttachPoint a("");
+  a.provider = "uprobe";
+  a.target = "/bin/sh";
+  a.func = "cpp_man*";
+  a.need_expansion = true;
+  ast::AttachPointList attach_points = { &a };
+  ast::Probe probe(&attach_points, nullptr, nullptr);
+
+  auto bpftrace = get_strict_mock_bpftrace();
+  EXPECT_CALL(*bpftrace, extract_func_symbols_from_path("/bin/sh")).Times(1);
+
+  ASSERT_EQ(0, bpftrace->add_probe(probe));
+  ASSERT_EQ(2U, bpftrace->get_probes().size());
+  ASSERT_EQ(0U, bpftrace->get_special_probes().size());
+  check_uprobe(bpftrace->get_probes().at(0),
+               "/bin/sh",
+               "_Z11cpp_mangledi",
+               "uprobe:/bin/sh:cpp_man*");
+  check_uprobe(bpftrace->get_probes().at(1),
+               "/bin/sh",
+               "_Z11cpp_mangledv",
+               "uprobe:/bin/sh:cpp_man*");
+}
+
 TEST(bpftrace, add_probes_usdt)
 {
-  ast::AttachPoint a("usdt", "/bin/sh", "prov1", "mytp", false);
+  ast::AttachPoint a("");
+  a.provider = "usdt";
+  a.target = "/bin/sh";
+  a.ns = "prov1";
+  a.func = "mytp";
+  a.usdt.num_locations = 1;
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -339,7 +511,13 @@ TEST(bpftrace, add_probes_usdt)
 
 TEST(bpftrace, add_probes_usdt_wildcard)
 {
-  ast::AttachPoint a("usdt", "/bin/sh", "prov*", "tp*", true);
+  ast::AttachPoint a("");
+  a.provider = "usdt";
+  a.target = "/bin/sh";
+  a.ns = "prov*";
+  a.func = "tp*";
+  a.usdt.num_locations = 1;
+  a.need_expansion = true;
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -362,7 +540,13 @@ TEST(bpftrace, add_probes_usdt_wildcard)
 
 TEST(bpftrace, add_probes_usdt_empty_namespace)
 {
-  ast::AttachPoint a("usdt", "/bin/sh", "", "tp", true);
+  ast::AttachPoint a("");
+  a.provider = "usdt";
+  a.target = "/bin/sh";
+  a.ns = "";
+  a.func = "tp";
+  a.usdt.num_locations = 1;
+  a.need_expansion = true;
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -380,9 +564,45 @@ TEST(bpftrace, add_probes_usdt_empty_namespace)
              "usdt:/bin/sh:prov2:tp");
 }
 
+TEST(bpftrace, add_probes_usdt_duplicate_markers)
+{
+  ast::AttachPoint a("");
+  a.provider = "usdt";
+  a.target = "/bin/sh";
+  a.ns = "prov1";
+  a.func = "mytp";
+  a.usdt.num_locations = 3;
+  ast::AttachPointList attach_points = { &a };
+  ast::Probe probe(&attach_points, nullptr, nullptr);
+
+  StrictMock<MockBPFtrace> bpftrace;
+
+  ASSERT_EQ(0, bpftrace.add_probe(probe));
+  ASSERT_EQ(3U, bpftrace.get_probes().size());
+  ASSERT_EQ(0U, bpftrace.get_special_probes().size());
+  check_usdt(bpftrace.get_probes().at(0),
+             "/bin/sh",
+             "prov1",
+             "mytp",
+             "usdt:/bin/sh:prov1:mytp");
+  check_usdt(bpftrace.get_probes().at(1),
+             "/bin/sh",
+             "prov1",
+             "mytp",
+             "usdt:/bin/sh:prov1:mytp");
+  check_usdt(bpftrace.get_probes().at(2),
+             "/bin/sh",
+             "prov1",
+             "mytp",
+             "usdt:/bin/sh:prov1:mytp");
+}
+
 TEST(bpftrace, add_probes_tracepoint)
 {
-  ast::AttachPoint a("tracepoint", "sched", "sched_switch", true);
+  ast::AttachPoint a("");
+  a.provider = "tracepoint";
+  a.target = "sched";
+  a.func = "sched_switch";
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -398,7 +618,11 @@ TEST(bpftrace, add_probes_tracepoint)
 
 TEST(bpftrace, add_probes_tracepoint_wildcard)
 {
-  ast::AttachPoint a("tracepoint", "sched", "sched_*", true);
+  ast::AttachPoint a("");
+  a.provider = "tracepoint";
+  a.target = "sched";
+  a.func = "sched_*";
+  a.need_expansion = true;
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -419,7 +643,11 @@ TEST(bpftrace, add_probes_tracepoint_wildcard)
 
 TEST(bpftrace, add_probes_tracepoint_wildcard_no_matches)
 {
-  ast::AttachPoint a("tracepoint", "typo", "typo_*", true);
+  ast::AttachPoint a("");
+  a.provider = "tracepoint";
+  a.target = "typo";
+  a.func = "typo_*";
+  a.need_expansion = true;
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -435,7 +663,10 @@ TEST(bpftrace, add_probes_tracepoint_wildcard_no_matches)
 
 TEST(bpftrace, add_probes_profile)
 {
-  ast::AttachPoint a("profile", "ms", 997);
+  ast::AttachPoint a("");
+  a.provider = "profile";
+  a.target = "ms";
+  a.freq = 997;
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -451,7 +682,10 @@ TEST(bpftrace, add_probes_profile)
 
 TEST(bpftrace, add_probes_interval)
 {
-  ast::AttachPoint a("interval", "s", 1);
+  ast::AttachPoint a("");
+  a.provider = "interval";
+  a.target = "s";
+  a.freq = 1;
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -467,7 +701,10 @@ TEST(bpftrace, add_probes_interval)
 
 TEST(bpftrace, add_probes_software)
 {
-  ast::AttachPoint a("software", "faults", 1000);
+  ast::AttachPoint a("");
+  a.provider = "software";
+  a.target = "faults";
+  a.freq = 1000;
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -483,7 +720,10 @@ TEST(bpftrace, add_probes_software)
 
 TEST(bpftrace, add_probes_hardware)
 {
-  ast::AttachPoint a("hardware", "cache-references", 1000000);
+  ast::AttachPoint a("");
+  a.provider = "hardware";
+  a.target = "cache-references";
+  a.freq = 1000000;
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -499,7 +739,9 @@ TEST(bpftrace, add_probes_hardware)
 
 TEST(bpftrace, invalid_provider)
 {
-  ast::AttachPoint a("lookatme", "invalid");
+  ast::AttachPoint a("");
+  a.provider = "lookatme";
+  a.func = "invalid";
   ast::AttachPointList attach_points = { &a };
   ast::Probe probe(&attach_points, nullptr, nullptr);
 
@@ -568,7 +810,7 @@ TEST(bpftrace, sort_by_key_int)
 {
   StrictMock<MockBPFtrace> bpftrace;
 
-  std::vector<SizedType> key_args = { SizedType(Type::integer, 8) };
+  std::vector<SizedType> key_args = { CreateUInt64() };
   std::vector<std::pair<std::vector<uint8_t>, std::vector<uint8_t>>> values_by_key =
   {
     key_value_pair_int({2}, 12),
@@ -592,9 +834,9 @@ TEST(bpftrace, sort_by_key_int_int)
   StrictMock<MockBPFtrace> bpftrace;
 
   std::vector<SizedType> key_args = {
-    SizedType(Type::integer, 8),
-    SizedType(Type::integer, 8),
-    SizedType(Type::integer, 8),
+    CreateUInt64(),
+    CreateUInt64(),
+    CreateUInt64(),
   };
   std::vector<std::pair<std::vector<uint8_t>, std::vector<uint8_t>>> values_by_key =
   {
@@ -625,7 +867,7 @@ TEST(bpftrace, sort_by_key_str)
   StrictMock<MockBPFtrace> bpftrace;
 
   std::vector<SizedType> key_args = {
-    SizedType(Type::string, STRING_SIZE),
+    CreateString(STRING_SIZE),
   };
   std::vector<std::pair<std::vector<uint8_t>, std::vector<uint8_t>>> values_by_key =
   {
@@ -652,9 +894,9 @@ TEST(bpftrace, sort_by_key_str_str)
   StrictMock<MockBPFtrace> bpftrace;
 
   std::vector<SizedType> key_args = {
-    SizedType(Type::string, STRING_SIZE),
-    SizedType(Type::string, STRING_SIZE),
-    SizedType(Type::string, STRING_SIZE),
+    CreateString(STRING_SIZE),
+    CreateString(STRING_SIZE),
+    CreateString(STRING_SIZE),
   };
   std::vector<std::pair<std::vector<uint8_t>, std::vector<uint8_t>>> values_by_key =
   {
@@ -685,8 +927,8 @@ TEST(bpftrace, sort_by_key_int_str)
   StrictMock<MockBPFtrace> bpftrace;
 
   std::vector<SizedType> key_args = {
-    SizedType(Type::integer, 8),
-    SizedType(Type::string, STRING_SIZE),
+    CreateUInt64(),
+    CreateString(STRING_SIZE),
   };
   std::vector<std::pair<std::vector<uint8_t>, std::vector<uint8_t>>> values_by_key =
   {
@@ -712,6 +954,45 @@ TEST(bpftrace, sort_by_key_int_str)
   EXPECT_THAT(values_by_key, ContainerEq(expected_values));
 }
 
+#ifdef HAVE_LIBBPF_BTF_DUMP
+
+#include "btf_common.h"
+
+class bpftrace_btf : public test_btf
+{
+};
+
+void check_kfunc(Probe &p, ProbeType type, const std::string &name)
+{
+  EXPECT_EQ(type, p.type);
+  EXPECT_EQ(name, p.name);
+}
+
+TEST_F(bpftrace_btf, add_probes_kfunc)
+{
+  ast::AttachPoint a("");
+  a.provider = "kfunc";
+  a.target = "func_1";
+  ast::AttachPoint b("");
+  b.provider = "kretfunc";
+  b.target = "func_1";
+  ast::AttachPointList attach_points = { &a, &b };
+  ast::Probe probe(&attach_points, nullptr, nullptr);
+
+  StrictMock<MockBPFtrace> bpftrace;
+
+  ASSERT_EQ(0, bpftrace.add_probe(probe));
+  ASSERT_EQ(2U, bpftrace.get_probes().size());
+  ASSERT_EQ(0U, bpftrace.get_special_probes().size());
+
+  check_kfunc(bpftrace.get_probes().at(0), ProbeType::kfunc, "kfunc:func_1");
+  check_kfunc(bpftrace.get_probes().at(1),
+              ProbeType::kretfunc,
+              "kretfunc:func_1");
+}
+
+#endif // HAVE_LIBBPF_BTF_DUMP
+
 } // namespace bpftrace
 } // namespace test
 } // namespace bpftrace
